/*
    *
    * This file is a part of CorePDF.
    * A PDF viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include "corepdf.h"
#include "ui_corepdf.h"
#include "qpdfpagenavigation.h"

#include <QtConcurrent/QtConcurrentRun>
#include <QApplication>

#include <cprime/pinit.h>
#include <cprime/shareit.h>
#include <cprime/themefunc.h>
#include <cprime/appopenfunc.h>


corepdf::corepdf(QWidget *parent) :QWidget(parent),ui(new Ui::corepdf) {

    ui->setupUi(this);

    loadSettings();
    startsetup();
}

corepdf::~corepdf() {

    delete ui;
    delete PdfDoc;
    delete PdfWidget;
}


/**
// * @brief Setup ui elements
 */
void corepdf::startsetup() {

	ui->spinBox->setRange( 0, 0 );
	ui->spinBox->setPrefix( "Page " );
	ui->spinBox->setSuffix( " of 0" );

	ui->tools->setFocusPolicy( Qt::NoFocus );
	ui->zoomCB->setFocusPolicy( Qt::NoFocus );
	ui->spinBox->setFocusPolicy( Qt::NoFocus );
	ui->pageModeCombo->setFocusPolicy( Qt::NoFocus );

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    // all toolbuttons icon size in toolBar
    for (QToolButton *b : ui->toolBar->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(toolBarIconSize);
        }
    }

    ui->page->setCurrentIndex(0);
    ui->sideView->setVisible( false );
    ui->tools->setEnabled(false);
    ui->rotate->setEnabled(false);

    // set window size
    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    } else {
        this->resize(800, 500);
    }

    connect(ui->addFiles, SIGNAL(clicked()), this, SLOT(openFileDialog()));
    connect(ui->open, SIGNAL(clicked()), this, SLOT(openFileDialog()));

    // set stylesheet
    ui->sideView->setStyleSheet( CPrime::ThemeFunc::getSideViewStyleSheet() );
    ui->sideView->setFixedWidth( ( int )sm->value( "CoreApps", "SideViewWidth", 220 ) );

    PdfWidget = new QPdfView( this );
    PdfWidget->setPageMode( QPdfView::MultiPage );
    PdfWidget->setZoomMode( QPdfView::FitToWidth );
    ui->u->addWidget(PdfWidget);

    connect( PdfWidget->pageNavigation(), &QPdfPageNavigation::currentPageChanged, this, &corepdf::updateSpinBox );
};

/**
 * @brief Loads application settings
 */
void corepdf::loadSettings()
{
    sideViewIconSize = sm->value( "CoreApps", "ViewIconSize", QSize( 24, 24 ) );
    toolBarIconSize = sm->value( "CoreApps", "ToolBarIconSize", QSize( 24, 24 ) );
    touch = sm->value( "CoreApps", "TouchMode", false );
}

/**
 * @brief Sends file to library
 */
void corepdf::loadFile( const QString &file )
{
    workFilePath = file;
    ui->page->setCurrentIndex(1);

    PdfDoc = new QPdfDocument( file );
    PdfDoc->load();
    PdfWidget->setDocument( PdfDoc );
    ui->fileName->setText( PdfDoc->title().isEmpty() ? PdfDoc->pdfName() : PdfDoc->title() );
    ui->tools->setEnabled( true );

    ui->zoomCB->setCurrentIndex( sm->value( "CorePDF", "ZoomMode" ) );
    // PdfWidget->setZoomFactor( ( int )sm->value( "CorePDF", "ZoomFactor" ) * 1.0 / 100 );
    ui->pageModeCombo->setCurrentIndex( sm->value( "CorePDF", "PageMode" ) );

	ui->spinBox->setRange( 1, PdfDoc->pageCount() );
	ui->spinBox->setValue( 1 );
    ui->spinBox->setSuffix( QString( " of %1" ).arg( PdfDoc->pageCount() ) );
}

void corepdf::closeEvent( QCloseEvent *event )
{
    event->ignore();
    CPrime::InfoFunc::saveToRecent( "corepdf", workFilePath );
    PdfWidget->close();
    event->accept();
}

void corepdf::on_tools_clicked()
{
    if ( ui->sideView->isVisible() ) {
        ui->sideView->setVisible( false );
    } else {
        ui->sideView->setVisible( true );
    }
}

void corepdf::on_shareIt_clicked()
{
    if ( !workFilePath.isNull() ) {
        shareit sit;
        sit.callShareIt( this, workFilePath );
    }
}

void corepdf::on_pinIt_clicked()
{
    if ( !workFilePath.isNull() ) {
        pinit pit;
        pit.callPinIt( this, workFilePath );
    }
}

void corepdf::openFileDialog()
{
	QString fn = QFileDialog::getOpenFileName(
		this,
		"CorePDF - Open File",
		QDir::homePath(),
		"PDF Files (*.pdf);;All Files (*.*)"
	);

	if ( not fn.isEmpty() )
		loadFile( fn );
}

void corepdf::on_openmetadataApp_clicked() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::MetadataViewer, workFilePath, this );
}

void corepdf::on_savePageAsPic_clicked() {

	int pageNo = PdfWidget->pageNavigation()->currentPage();
	QSizeF pSize = PdfDoc->pageSize( pageNo );

	QInputDialog *zoomDlg = new QInputDialog( this );
	zoomDlg->setInputMode( QInputDialog::IntInput );
	zoomDlg->setLabelText(
		QString(
			"Size of page %1 is %2 x %3 px. Please set the zoom factor below if necesssary."
		).arg( pageNo + 1 ).arg( pSize.width() ).arg( pSize.height() )
	);
	zoomDlg->setIntRange( 10, 400 );
	zoomDlg->setIntStep( 10 );
	zoomDlg->setIntValue( 100 );

	Q_FOREACH( QSpinBox *sb, zoomDlg->findChildren<QSpinBox *>() ) {
		if ( sb )
			sb->setSuffix( " %" );
	}

	if ( zoomDlg->exec() == QDialog::Accepted ) {
		qreal zoomFactor = zoomDlg->intValue() * 1.0 / 100.0;
		QPdfDocumentRenderOptions opts;
		opts.setRotation( m_rotation );
		QImage image = PdfDoc->renderPage( PdfWidget->pageNavigation()->currentPage(), zoomFactor, opts );

		if ( not image.isNull() ) {
			QStringList fmts;
			Q_FOREACH( QByteArray fmtba, QImageWriter::supportedImageFormats() ) {
				QString fmt = QString::fromLatin1( fmtba );
				fmts << fmt.toUpper() + " Image (*." + fmt + ")";
			}

			QString fn = QFileDialog::getSaveFileName(
				this,
				"CorePDF - Save Current Page",
				QDir::homePath(),
				fmts.join( ";;" ) + ";;All Files (*.*)"
			);

			bool ret = false;
			if ( not fn.isEmpty() ) {
				ret = image.save( fn );
			}

			if ( ret )
				CPrime::InfoFunc::messageEngine( "Image saved successfully.", CPrime::MessageType::Info, this );

			else
				CPrime::InfoFunc::messageEngine( "Failed to save page as image.", CPrime::MessageType::Warning, this );
		}
	}
};

void corepdf::on_spinBox_valueChanged( int pageNo ) {

	if ( pageNo > 0 )
		if ( pageNo <= PdfDoc->pageCount() )
			PdfWidget->pageNavigation()->setCurrentPage( pageNo - 1 );

		else
			PdfWidget->pageNavigation()->setCurrentPage( PdfDoc->pageCount() - 1 );

	else
		PdfWidget->pageNavigation()->setCurrentPage( 0 );
};

void corepdf::on_zoomCB_currentIndexChanged( int idx ) {

	switch( idx ) {
		case 0: {
			PdfWidget->setZoomMode( QPdfView::FitToWidth );
            sm->setValue( "CorePDF", "ZoomMode", 0 );
			break;
		}

		case 1: {
			PdfWidget->setZoomMode( QPdfView::FitInView );
            sm->setValue( "CorePDF", "ZoomMode", 1 );
			break;
		}

		default: {
			PdfWidget->setZoomMode( QPdfView::CustomZoom );
			qreal zoom = ui->zoomCB->currentText().replace( "%", "" ).toInt() * 1.0 / 100;
            sm->setValue( "CorePDF", "ZoomMode", ui->zoomCB->currentIndex() );
			PdfWidget->setZoomFactor( zoom );
		}
	}
}

void corepdf::on_rotate_clicked() {

	int rotation = ( int )m_rotation + 1;
	m_rotation = ( QPdf::Rotation )( rotation % 4 );
	// PdfWidget->setRotation( m_rotation );
}

void corepdf::on_pageModeCombo_currentIndexChanged( int idx ) {

	if ( not PdfDoc or not PdfWidget )
		return;

	switch( idx ) {
		case 0: {
			PdfWidget->setPageMode( QPdfView::PageMode::SinglePage );
			sm->setValue( "CorePDF", "PageMode", 0 );
			break;
		}

		case 1: {
			PdfWidget->setPageMode( QPdfView::PageMode::MultiPage );
			sm->setValue( "CorePDF", "PageMode", 1 );
			break;
		}
	}
}

void corepdf::updateSpinBox( int pageNo )
{
	ui->spinBox->setValue( pageNo + 1 );
}
